#ifndef PostgresDatabase_H
#define PostgresDatabase_H

/*
 * This class Postgres::Database extends PgDatabase and consists solely
 * of this header file with inline implementations. Usage is identical
 * to PgDatabase, except that the user need not check for error conditions.
 * An exception (Postgres::Exception) is thrown instead.
 * See the file testexcep.cxx for an example of the usage.
 *
 * Author: Tom Vijlbrief, (tom.vijlbrief@knoware.nl)
 * Wed Apr 19 18:28:35 CEST 2000
 */

#include <libpq++/pgdatabase.h>
#include <libpq++/pglobject.h>

#include <exception>

namespace Postgres {

class Exception : public std::exception {
   const char* _reason;
public:
   Exception(const char *s) : _reason(s) { }
   virtual const char* what() const;
};

inline const char* Exception::what() const { return _reason; }


class Database : public PgDatabase {
#if 0 // I'm not sure if different exception types are usefull
      // For now we don't use them to keep the generated code smaller...
  class ConnectionException: public Exception {
  public:
    ConnectionException(const char *s) : Exception(s) {}
  };
  class ExecException: public Exception {
  public:
    ExecException(const char *s) : Exception(s) {}
  };
#else
  typedef Exception ConnectionException;
  typedef Exception ExecException;
#endif
public:
  Database(const char *s);
  ExecStatusType Exec(const char *s);

  class TransAction {
  protected:
    Database& _db;
  public:
    TransAction(Database &db);
    ~TransAction();
  };

  class Cursor : public TransAction {
    string    _name;
  public:
    Cursor(Database &db, const string &q, bool binary= false,
      const string &name= "defname");
    ~Cursor();

    void Close();
    void Fetch(const char * dir= "FORWARD"); // Fetch All
    void Fetch(unsigned num, const char * dir = "FORWARD");
    void Fetch(string num, const char * dir = "FORWARD");
  };
  
};

inline Database::Database(const char *s) : PgDatabase(s) {
    if (ConnectionBad())
      throw ConnectionException(ErrorMessage());
}

inline ExecStatusType Database::Exec(const char *s) {
    ExecStatusType stat= PgDatabase::Exec(s);
    if (stat != PGRES_COMMAND_OK
    && stat != PGRES_TUPLES_OK
    && stat != PGRES_COPY_OUT
    && stat != PGRES_COPY_IN
    )
      throw ExecException(ErrorMessage());
    return stat;
}

inline Database::TransAction::TransAction(Database &db) : _db(db)
{ _db.Exec("begin"); }

inline Database::TransAction::~TransAction()
{ _db.Exec("end"); }


inline Database::Cursor::Cursor(Database &db,
  const string &query, bool binary, const string &name)
	: TransAction(db), _name(name)
{
  string cmd = "DECLARE " + name;
  if (binary)
	cmd += " BINARY";
  cmd += " CURSOR FOR " + query;
  _db.Exec(cmd.c_str());
}

inline Database::Cursor::~Cursor()
{
  Close();
}

inline void Database::Cursor::Fetch(const char * dir)
{
  Fetch("ALL", dir);
}

inline void Database::Cursor::Fetch(unsigned num, const char * dir)
{
  Fetch(PgDatabase::IntToString(num), dir);
}

inline void Database::Cursor::Fetch(string num, const char * dir)
{
  string cmd = "FETCH " + string(dir) + " " + num + " IN " + _name;
  _db.Exec( cmd.c_str() );
}

inline void Database::Cursor::Close()
{
  string cmd = "CLOSE " + _name;
  _db.Exec(cmd.c_str());
}

} // namespace Postgres

#endif // PostgresDatabase_H
