/*-------------------------------------------------------------------------
 *
 *   FILE
 *	pglobject.cc
 *
 *   DESCRIPTION
 *      implementation of the PgLargeObject class.
 *   PgLargeObject encapsulates a frontend to backend connection
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * IDENTIFICATION
 *	  $Header: /usr/local/cvsroot/pgsql/src/interfaces/libpq++/pglobject.cc,v 1.5 1999/05/30 15:17:58 tgl Exp $
 *
 *-------------------------------------------------------------------------
 */
 
extern "C" {
#include "libpq/libpq-fs.h"
}

#include "pglobject.h"

#include <exception>

class LargeObjectException : public std::exception {
   const string _reason;
public:
   LargeObjectException(string s) : _reason(s) { }
   virtual const char* what() const;
};

inline const char* LargeObjectException::what() const
  { return _reason.c_str(); }


// ****************************************************************
//
// PgLargeObject Implementation
//
// ****************************************************************
// default constructor
// creates a large object in the default database
// See PQconnectdb() for conninfo usage
PgLargeObject::PgLargeObject(PgDatabase& db)
	: pdb(db)
{
  Init();
  Create();
  Open();
}

// constructor
// open an existing large object in the default database
// See PQconnectdb() for conninfo usage
PgLargeObject::PgLargeObject(Oid lobjId, PgDatabase& db)
	: pdb(db)
{
  Init(lobjId);
  if ( !pgObject ) {
	Create();
  }
  Open();
}

// destructor -- closes large object
PgLargeObject::~PgLargeObject()
{
  Close();
}

// PgLargeObject::Init
// Initialize the variables
void PgLargeObject::Init(Oid lobjId)
{
  pgFd = -1;
  pgObject = lobjId;
  pdb.Exec("begin");
}

// PgLargeObject::create
// create large object and check for errors
void PgLargeObject::Create()
{
  // Create the object
  pgObject = lo_creat(pdb.GetCon(), INV_READ|INV_WRITE);
  
  // Check for possible errors
  if (!pgObject)
    throw LargeObjectException("PgLargeObject: can't create large object");
}

// PgLargeObject::open
// open large object and check for errors
void PgLargeObject::Open()
{
  // Open the object
  pgFd = lo_open(pdb.GetCon(), pgObject, INV_READ|INV_WRITE);
  
  // Check for possible errors
  if (pgFd < 0) {
    string objStr( pdb.IntToString(pgObject) );
    throw LargeObjectException("PgLargeObject: can't create large object "
	+ objStr);
  }
}

// PgLargeObject::unlink
// destruct large object and delete from it from the database
void PgLargeObject::Unlink()
{
  Close();

  // Unlink the object
  int temp = lo_unlink(pdb.GetCon(), pgObject);
  
  // Initialize the large object upon success
  if (!temp) {
    throw LargeObjectException("PgLargeObject: can't unlink large object");
  }
}



void PgLargeObject::Close()
{ 
  if (pgFd >= 0) {
    lo_close(pdb.GetCon(), pgFd); 
    pgFd= -1;
  }
  pdb.Exec("end");
}


int PgLargeObject::Read(char* buf, int len)
{ 
  return lo_read(pdb.GetCon(), pgFd, buf, len); 
}


int PgLargeObject::Write(const char* buf, int len)
{ 
  return lo_write(pdb.GetCon(), pgFd, (char*)buf, len); 
}


int PgLargeObject::LSeek(int offset, int whence)
{ 
  return lo_lseek(pdb.GetCon(), pgFd, offset, whence); 
}


int PgLargeObject::Tell()
{ 
  return lo_tell(pdb.GetCon(), pgFd); 
}


Oid PgLargeObject::Import(const char* filename) 
{ 
  return pgObject = lo_import(pdb.GetCon(), (char*)filename); 
}


int PgLargeObject::Export(const char* filename) 
{ 
  return lo_export(pdb.GetCon(), pgObject, (char*)filename); 
}
